from loonybin import Tool

class Combine(Tool):
    
    def getName(self):
        return 'Machine Translation/Grammars and Tables/Chaski (Hadoop)/Phrase Extract Prep'
    
    def getDescription(self):
        return "Provides a tool to convert Moses-generated alignment files to Chaski format"
    
    def getRequiredPaths(self):
        return ['chaski']

    def getParamNames(self):
        return [ ('sentencesPerSplit','number of sentences to put in each "split" (chunked file on HDFS)') ]
    
    def getInputNames(self, params):
        return [ ('hdfs:srcCorpus','Source corpus'),
                 ('hdfs:tgtCorpus','Target corpus'),
                 ('hdfs:alignments','The alignment generated by Moses') ]

    def getOutputNames(self, params):
        return [ ('hdfs:alignedCorpus', 'The alignment file that is readable by Chaski phrase extraction') ]
    
    def getCommands(self, params, inputs, outputs):
        cmd = ('hadoop jar chaski-0.0-latest.jar preproc --overwrite true --src %(hdfs:srcCorpus)s --tgt %(hdfs:tgtCorpus)s --align %(hdfs:alignments)s '%inputs+
                 '--corpus %(hdfs:alignedCorpus)s'%outputs+
                 '--splits %(sentencesPerSplit)s --src-on-hdfs true --tgt-on-hdfs true --aln-on-hdfs true --out-on-hdfs true'%params)
        
        return [cmd,
                '',
                '']

    def getPostAnalyzers(self, params, inputs, outputs):
        return [ ]

if __name__ == '__main__':
    import sys
    t = Combine()
    t.handle(sys.argv)
