#!/usr/bin/env python
from loonybin import Tool

class MultiMetric(Tool):
    
    def getName(self):
        return "Machine Translation/Scoring/MultiMetric"
        
    def getDescription(self):
        return "Runs BLEU, METEOR, and TER on a set of hypotheses after passing them through Greg's detokenization script"
    
    def getRequiredPaths(self):
        return ['scoring-scripts', 'joshua-scripts']

    def getInputNames(self, params):
        return [ ('refs', ''),
                ('hyps','') ]

    def getOutputNames(self, params):
        return [ ('scoresOut',''),
                 ('meteorOut', ''),
                 ('bleuOut', ''),
                 ('terOut','') ]
    
    def getCommands(self, params, inputs, outputs):
        # multimetric.sh refs hyps scoresOut
        commands = []
        commands.append('score.rb --hyp-detok %(hyps)s --refs-laced %(refs)s --output out.'%inputs)
        commands.append('mv out.scores %(scoresOut)s'%outputs)
        commands.append('mv out.meteor_out %(meteorOut)s'%outputs)
        commands.append('mv out.bleu_out %(bleuOut)s'%outputs)
        commands.append('mv out.ter_out.sum %(terOut)s'%outputs)
        return commands

    def getPostAnalyzers(self, params, inputs, outputs):
        cmd = ('multimetric-analyze.py %(meteorOut)s %(bleuOut)s %(terOut)s'%outputs)
        return [ cmd ]

import sys
t = MultiMetric()
t.handle(sys.argv)
