#!/usr/bin/env python
from loonybin import Tool
from libmoses import *

class LearnLex(Tool):

    def getName(self):
        return 'Machine Translation/Grammars and Tables/Local/Learn Lexicalized Reordering Model'
    
    def getDescription(self):
        return "Extracts phrase instances using the standard Moses script"
    
    def getRequiredPaths(self):
        return ['moses']

    def getParamNames(self):
        return [ ('roModelType','one of msd-bidirectional-fe (default), msd-bidirectional-e, msd-fe, msd-f, '+
                                'monotonicity-bidirectional-fe, monotonicity-bidirectional-f, monotonicity-fe, monotonicity-f ') ]

    def getInputNames(self, params):
        return [ ('phraseInstanceOrientations', '?') ]

    def getOutputNames(self, params):   
        return [ ('scoredReorderingTable','?') ]

    def getCommands(self, params, inputs, outputs):     
        commands = getDummyMosesCommands()
        # TODO: Symlink both directions of lexicons and extract files to roots
        # Double check order of inv and lexE2F
        commands.append('ln -s %(phraseInstanceOrientations)s ext.o'%inputs)
        commands.append(
                './scripts/training/train-factored-phrase-model.perl --first-step 7 --last-step 7 --bin-dir dummyBin --f f --e e --model-dir .'+
                ' --reordering %(roModelType)s'%params+
                ' --extract-file ext'%inputs+
                ' --reordering-table ro')
        commands.append('gunzip ro.gz # Unzip for now until Moses supports not zipping or LoonyBin supports gzip')
        commands.append('mv ro %(scoredReorderingTable)s'%outputs)
        return commands

import sys
t = LearnLex()
t.handle(sys.argv)
