from loonybin import Tool

class Moses(Tool):
    
    def getName(self):
        return 'Machine Translation/Decoders/Moses'
    
    def getDescription(self):
        return """
Takes a translation grammar, glue grammar, language model, and lexicalized 
distortion model and produces n-best translation hypotheses using Jon's working
copy of Joshua.
"""
    
    def getRequiredPaths(self):
        return ['moses']

    # Decoder params shared with MERT
    def getSharedParams(self):
         return [ ('lmOrder', '?'),
                  ('roModelType','?'),
                  ('nRoFeatures','?'),
                  ('nPtFeatures','?'),
                  ('ttableLimit','?'),
                  ('distortionLimit','?'),
                  ('nBestSize','?'), ]

    def getParamNames(self):
        params = self.getSharedParams()
        return params
   
    # Decoder params shared with MERT
    def getSharedInputs(self):
        return [ ('lmFile', 'ARPA format language model'),
                 ('ptFile','Moses format phrase table'),
                 ('roFile','Moses format lexicalized reordering table') ]

    def getInputNames(self, params):
        inputs = self.getSharedInputs()
        inputs.extend([ ('fSentsIn', '"French" sentences for which n-best hypotheses will be produced'),
                        ('optimizedConfigFile', 'Final configuration file resulting from MERT containing optimized feature weights') ])
        return inputs

    def getOutputNames(self, params):
        return [ ('eNBestOut', '"English" n-best hypotheses in Joshua format for the fSentsIn') ]
    
    def makeConfigFileCommands(self, params, inputs, configFileName):
           
        # TODO: Read LM order from arpa file?
        
        mosesConfig = []
        mosesConfig.append('[input-factors]')
        mosesConfig.append('0')
        mosesConfig.append('')
        mosesConfig.append('[mapping]')
        mosesConfig.append('0 T 0')
        mosesConfig.append('')
        mosesConfig.append('[ttable-file]')
        mosesConfig.append('0 0 %(nPtFeatures)s '%params + '%(ptFile)s'%inputs)
        mosesConfig.append('')
        mosesConfig.append('[lmodel-file]')
        mosesConfig.append('0 0 %(lmOrder)s'%params + ' %(lmFile)s'%inputs)
        mosesConfig.append('')
        mosesConfig.append('[distortion-file]')
        mosesConfig.append('0-0 %(roModelType)s %(nRoFeatures)s'%params + ' %(roFile)s'%inputs)
        mosesConfig.append('')
        mosesConfig.append('[ttable-limit]')
        mosesConfig.append('%(ttableLimit)s'%params)
        mosesConfig.append('0') # load all elements
        mosesConfig.append('')
        mosesConfig.append('[distortion-limit]')
        mosesConfig.append('%(distortionLimit)s'%params)
        mosesConfig.append('')
        
        commands = []
        for line in mosesConfig:
            commands.append('echo "' + line +  '" >> ' + configFileName)
            
        # get model weights from paramFile (if paramFile defined -- not true for tuning)
        if 'optimizedConfigFile' in inputs:
            # Extract only lines pertaining to weights from the optimized config file
            commands.append(r"awk '{if(/\[weight/) {inWeight=1} if(inWeight) {print} if(/^$/) {inWeight=0} }' %(optimizedConfigFile)s"%inputs+
                                ' >> ' + configFileName)
        
            
        return commands
    
    def getCommands(self, params, inputs, outputs):
     
        commands = self.makeConfigFileCommands(params, inputs, 'moses.ini')
        cmd1 = ('./moses-cmd/src/moses -f moses.ini -i %(fSentsIn)s '%inputs +
                '-n-best-list %(eNBestOut)s '%outputs + 
                '%(nBestSize)s distinct'%params)
        commands.append(cmd1)
        return commands

    def getPostAnalyzers(self, params, inputs, outputs):
        # TODO: More detailed analysis of output
        #cmd = ('extract_feature_data.py %(eNBestOut)s'%outputs +
        #       ' %(optimizedConfigFile)s'%inputs)
        return [ ]

if __name__ == '__main__':
    import sys
    t = Moses()
    t.handle(sys.argv)
