from loonybin import Tool
from Moses import Moses

class MosesMert(Tool):
    
    def __init__(self):
        self.decoder = Moses()
    
    def getName(self):
        return 'Machine Translation/Tuning/Moses MERT'
    
    def getDescription(self):
        return "Tunes a translation system, decoding with Moses and using Moses' implementation of MERT"
    
    def getRequiredPaths(self):
        return ['moses']

    def getParamNames(self):
        params = self.decoder.getSharedParams()
        return params

    def getInputNames(self, params):
        inputs = self.decoder.getSharedInputs()
        inputs.extend([ ('fCorpus','?'),
                        ('refs','?') ])
        return inputs

    def getOutputNames(self, params):
        return [ ('finalConfigFile','?') ]
    
    def getCommands(self, params, inputs, outputs):

        # Create base moses.ini on the fly
        commands = self.decoder.makeConfigFileCommands(params, inputs, 'moses.ini')
        
        # leave off all mentions of features?
        
        commands.append('./scripts/training/absolutize_moses_model.pl moses.ini > moses.abs.ini')
        commands.append('./scripts/training/mert-moses-new.pl --rootdir `pwd`/scripts --mertdir `pwd`/mert --input `pwd`/%(fCorpus)s --refs `pwd`/%(refs)s --decoder `pwd`/moses-cmd/src/moses --config `pwd`/moses.abs.ini 2>&1 | tee mert.log'%inputs)
        commands.append('ln -s ../mert-work/moses.ini %(finalConfigFile)s'%outputs)
        
        return commands
    
    def getPreAnalyzers(self, params, inputs):
        # TODO: Log entire config file
        # TODO: Make sure we don't have any blank inputs
        #cmd = ('AnalyzeParCorpus.sh %(fCorpusOut)s %(eCorpusOut)s'%outputs
        #       )
        return [ ]

    def getPostAnalyzers(self, params, inputs, outputs):
        # TODO: Make sure we have the same number of output hypotheses as inputs
        #cmd = ('extract_zmert_stats.pl zmert.stdout %(paramConfigFile)s'%inputs)
        return [ ]

if __name__ == '__main__':
    import sys
    t = MosesMert()
    t.handle(sys.argv)
    
