#!/usr/bin/env python
import sys
import urllib
import tarfile
import os

def filename(downloadDir, url):
    return downloadDir + '/' + url.rsplit('/', 1)[1].rsplit('?',1)[0]

def mkdir(path):
    if not os.path.exists(path):
        os.mkdir(path)

def reportProgress(count, blockSize, totalSize):
    print '.', #count, blockSize, totalSize

def retrieveFiles(deps, downloadDir, noRetrieve):
    print 'Retrieving files...'
    
    locs = dict()
    mkdir(downloadDir)
    for pathName, what, url, installer in deps:
        
        f = filename(downloadDir, url)
        locs[pathName] = f

        print 'Retriving %s from %s to %s'%(what,url,f)
        if noRetrieve:
            continue
        fido = urllib.FancyURLopener()
        try:
            fido.retrieve(url, f, reportProgress)
        except IOError as e:
            print >>sys.stderr, e
            sys.exit(1)
        print

    return locs

def installAll(deps, downloadLocs, installDir, pathsDir):

    mkdir(installDir)
    for pathName, what, url, installer in deps:
        os.chdir(installDir) # Don't accidentally end up in a new directory
        f = downloadLocs[pathName]
        print 'Installing %s to %s'%(what,installDir)
        installer(f, installDir, pathName, pathsDir)

def extractTarball(downloadedFile, installDir):
    tar = tarfile.open(downloadedFile, 'r:gz')
    tar.extractall(path=installDir)
    tar.close()

def writePathfile(pathsDir, pathName, path):
    mkdir(pathsDir)
    pathFile = '%s/%s.path'%(pathsDir, pathName)
    f = open(pathFile, 'w')
    f.write(path + '\n')
    f.close()
    print 'Wrote path file %s'%(pathFile)

def run(cmd):
    result = os.system(cmd)
    if result != 0:
        sys.exit(result)

def printUsage(deps):
    print >>sys.stderr, 'Usage: %s downloadDir installDir pathsDir'%(sys.argv[0])
    print >>sys.stderr, ''
    print >>sys.stderr, 'Available dependencies to install:'
    for pathName, what, url, func in deps:
        print >>sys.stderr, '%s\t%s'%(pathName, what)

def runInstaller(deps):

    if len(sys.argv) < 4:
        printUsage(deps)
    else:
        (downloadDir, installDir, pathsDir) = sys.argv[1:4]
        
        for opt in sys.argv[4:]:
            if opt.startswith('--without-'):
                exclude = opt[10:]
                print exclude
                deps = [d for d in deps if d[0] != exclude]
                
        downloadDir = os.path.abspath(downloadDir)
        installDir = os.path.abspath(installDir)
        pathsDir = os.path.abspath(pathsDir)

        if '--no-retrieve' in sys.argv[4:]:
            noRetrieve = True
        else:
            noRetrieve = False

        locs = retrieveFiles(deps, downloadDir, noRetrieve)
        installAll(deps, locs, installDir, pathsDir)
